﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using ArcGIS.Core.CIM;
using ArcGIS.Core.Data;
using ArcGIS.Core.Geometry;
using ArcGIS.Desktop.Catalog;
using ArcGIS.Desktop.Core;
using ArcGIS.Desktop.Editing;
using ArcGIS.Desktop.Extensions;
using ArcGIS.Desktop.Framework;
using ArcGIS.Desktop.Framework.Contracts;
using ArcGIS.Desktop.Framework.Dialogs;
using ArcGIS.Desktop.Framework.Threading.Tasks;
using ArcGIS.Desktop.Mapping;


namespace EditingWithUICustomizations
{
  internal class ConstructionToolWithAttachment_Chain : MapTool
  {
    public ConstructionToolWithAttachment_Chain()
    {
      IsSketchTool = true;
      UseSnapping = true;
      SketchType = SketchGeometryType.Point;
      //Gets or sets whether the sketch is for creating a feature and should use the CurrentTemplate.
      UsesCurrentTemplate = true;
    }

    /// <summary>
    /// Called when the sketch finishes. This is where we will create the sketch operation and then execute it.
    /// </summary>
    /// <param name="geometry">The geometry created by the sketch.</param>
    /// <returns>A Task returning a Boolean indicating if the sketch complete event was successfully handled.</returns>
    protected override Task<bool> OnSketchCompleteAsync(Geometry geometry)
    {
      if (CurrentTemplate == null || geometry == null)
        return Task.FromResult(false);

      return QueuedTask.Run(() =>
      {
        // Create an edit operation
        var createOperation = new EditOperation();
        createOperation.Name = string.Format("Create {0}", CurrentTemplate.Layer.Name);
        createOperation.SelectNewFeatures = true;

        // Queue feature creation

        // ** use a post execute Action to obtain the new objectId.   oid is a special defined variable
        long new_oid = -1;
        createOperation.Create(CurrentTemplate, geometry, oid => new_oid = oid);

        // execute the operation.
        bool result = createOperation.Execute();
        // check the return value
        if (result)
        {
          // ** create succceeded so chain a new operation from the original operation
          var chained_op = createOperation.CreateChainedOperation();
          // add the attachment using the new objectId and the chained EditOperation object
          chained_op.AddAttachment(CurrentTemplate.Layer, new_oid, Module1.FireStationFile);

          // execute the chained operation
          result = chained_op.Execute();
        }

        return result;
      });

    }
  }
}
